@@ -2,7 +2,7 @@
  * addr2uncompaddr: Support a function to format a given address to uncompressed one
  *                   (anti RFC 1884)
  *
- * Version:		$Id: addr2uncompaddr.c,v 0.07 2001/02/27 $
+ * Version:		$Id: addr2uncompaddr.c,v 0.09 2001/02/28 $
  * 
  * Author:		Peter Bieringer <pb@bieringer.de>
  *
@@ -39,18 +39,32 @@ void printhelp_addr2uncompaddr() {
  */
 int addr2uncompaddr(char *addrstring, char *resultstring) {
 	int retval = 1;
-	char cnt, *cp, *op;
+	char *addronlystring, cnt, *cp, *op;
+	int prefixlength = 0, flag_prefixlength = 0; 
 
 #ifdef DEBUG_addr2uncompaddr
 	fprintf(stderr, "addr2uncompaddr: Got input %s\n", addrstring);
 #endif
 
-	if (strstr(addrstring, "::")) {
+	/* save prefix length first, if available */
+    addronlystring = strtok (addrstring, "/");
+    cp = strtok (NULL, "/");
+	if ( cp != NULL ) {
+		prefixlength = atol(cp);
+		if (prefixlength < 0 || prefixlength > 128 ) {
+			sprintf(resultstring, "illegal prefix length");
+			retval = 1;
+		return (retval);
+		}
+		flag_prefixlength = 1; 
+	}
+
+	if (strstr(addronlystring, "::")) {
 #ifdef DEBUG_addr2uncompaddr
 		fprintf(stderr, "addr2uncompaddr: Found '::' in IPv6 address\n");
 #endif
 		cnt = 0;
-		cp = addrstring;
+		cp = addronlystring;
 		op = resultstring;
 	   	while (*cp) {
 			if (*cp == ':')	cnt += 1;
@@ -69,7 +83,7 @@ int addr2uncompaddr(char *addrstring, char *resultstring) {
 					*op++ = ':';
 				};
 		   		*op++ = '0';
-				while (cnt++ < 8) {
+				while (cnt++ < 7) {
 					*op++ = ':';
 					*op++ = '0';
 				}
@@ -77,6 +91,10 @@ int addr2uncompaddr(char *addrstring, char *resultstring) {
 		}
 		if (*(op-1)==':') *op++ = '0';
 		*op = '\0';
+
+		if ( flag_prefixlength == 1 ) {
+			sprintf(resultstring, "%s/%d", resultstring, prefixlength);
+		}
 #ifdef DEBUG_addr2uncompaddr
 		fprintf(stderr, "addr2uncompaddr: Result: %s\n", resultstring);
 #endif
